<?php

namespace Drupal\media_jove\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\media\Entity\MediaType;
use Drupal\media_jove\JoveUrlResolverInterface;
use Drupal\media_jove\Plugin\media\Source\Jove;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'jove_embed' formatter.
 *
 * @FieldFormatter(
 *   id = "jove_embed",
 *   label = @Translation("JoVE embedded content"),
 *   field_types = {
 *     "string_long",
 *   },
 * )
 */
class JoveEmbedFormatter extends FormatterBase implements ContainerFactoryPluginInterface {

  /**
   * JoVE URL resolver.
   *
   * @var \Drupal\media_jove\JoveUrlResolverInterface
   */
  protected $resolver;

  /**
   * {@inheritdoc}
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, JoveUrlResolverInterface $resolver) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);

    $this->resolver = $resolver;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('media_jove.url_resolver')
    );
  }

  /**
   * Builds a renderable array for a field value.
   *
   * @param \Drupal\Core\Field\FieldItemListInterface $items
   *   The field values to be rendered.
   * @param string $langcode
   *   The language that should be used to render the field.
   *
   * @return array
   *   A renderable array for $items, as an array of child elements keyed by
   *   consecutive numeric indexes starting from 0.
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $element = [];

    foreach ($items as $delta => $item) {
      $main_property = $item->getFieldDefinition()->getFieldStorageDefinition()->getMainPropertyName();
      $value = $item->{$main_property};

      if (empty($value)) {
        continue;
      }

      $id = $this->resolver->getVideoIdFromEmbedCode($value);
      $url = $this->resolver->getEmbedUrl($id);

      $query = $url->getOption('query');
      $query_variables = [
        'title' => 't',
        'description' => 'i',
        'authors' => 'a',
        'chapters' => 'chap',
        'autoplay' => 's',
      ];

      foreach ($query_variables as $option => $query_variable) {
        $query[$query_variable] = $this->getSetting($option) ? 1 : NULL;
      }

      // Get access code from original value of JoVe embed url.
      $embed_query_string = parse_url($value, PHP_URL_QUERY);
      // Parse the query string and get the parameters.
      parse_str($embed_query_string, $embed_query_params);
      if (!empty($embed_query_params['access'])) {
        // Add access code to url query, if it is specified.
        $query['access'] = $embed_query_params['access'];
      }

      $url->setOption('query', $query);

      $width = $this->getSetting('width');
      $height = $this->getSetting('height');

      // Handle width and height depending on video options.
      if ($this->getSetting('title')) {
        $height += 50;
      }
      if ($this->getSetting('authors')) {
        $height += 100;
      }
      if ($this->getSetting('description')) {
        $height += 150;
      }
      if ($this->getSetting('chapters')) {
        $width += 280;
      }

      $element[$delta] = [
        '#theme' => 'media_jove_video',
        '#uri' => $url->toString(),
        '#options' => [
          'iframe_width' => $width,
          'iframe_height' => $height,
          'media_id' => $item->getEntity()->id(),
          'video_id' => $id,
        ],
      ];
    }

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'width' => 460,
      'height' => 365,
      'title' => FALSE,
      'description' => FALSE,
      'authors' => FALSE,
      'chapters' => FALSE,
      'autoplay' => FALSE,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form = parent::settingsForm($form, $form_state);

    // Video width.
    $form['width'] = [
      '#title' => t('Width'),
      '#type' => 'textfield',
      '#default_value' => $this->getSetting('width'),
      '#element_validate' => [
        [$this, 'elementValidateWidthHeight'],
      ],
    ];

    // Video height.
    $form['height'] = [
      '#title' => t('Height'),
      '#type' => 'textfield',
      '#default_value' => $this->getSetting('height'),
      '#element_validate' => [
        [$this, 'elementValidateWidthHeight'],
      ],
    ];

    // Title option.
    $form['title'] = [
      '#title' => t('Display title'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('title'),
    ];

    // Description option.
    $form['description'] = [
      '#title' => t('Display description'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('description'),
    ];

    // Authors option.
    $form['authors'] = [
      '#title' => t('Display authors'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('authors'),
    ];

    // Chapters option.
    $form['chapters'] = [
      '#title' => t('Display chapters'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('chapters'),
    ];

    // Autoplay option.
    $form['autoplay'] = [
      '#title' => t('Autoplay video on load'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('autoplay'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();

    if ($this->getSetting('width') && $this->getSetting('height')) {
      $summary[] = $this->t('Size: %width x %height pixels', [
        '%width' => $this->getSetting('width'),
        '%height' => $this->getSetting('height'),
      ]);
    }
    elseif ($this->getSetting('width')) {
      $summary[] = $this->t('Width: %width pixels', [
        '%width' => $this->getSetting('width'),
      ]);
    }
    elseif ($this->getSetting('height')) {
      $summary[] = $this->t('Height: %height pixels', [
        '%height' => $this->getSetting('height'),
      ]);
    }

    $summary[] = $this->t('Display title: %display_title.', [
      '%display_title' => $this->getSetting('title') ? 'yes' : 'no',
    ]);

    $summary[] = $this->t('Display description: %display_description.', [
      '%display_description' => $this->getSetting('description') ? 'yes' : 'no',
    ]);

    $summary[] = $this->t('Display authors: %display_authors.', [
      '%display_authors' => $this->getSetting('authors') ? 'yes' : 'no',
    ]);

    $summary[] = $this->t('Display chapters: %display_chapters.', [
      '%display_chapters' => $this->getSetting('chapters') ? 'yes' : 'no',
    ]);

    $summary[] = $this->t('Autoplay video on load: %autoplay.', [
      '%autoplay' => $this->getSetting('autoplay') ? 'yes' : 'no',
    ]);

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public static function isApplicable(FieldDefinitionInterface $field_definition) {
    if ($field_definition->getTargetEntityTypeId() !== 'media') {
      return FALSE;
    }

    if (parent::isApplicable($field_definition)) {
      $media_type = $field_definition->getTargetBundle();

      if ($media_type) {
        $media_type = MediaType::load($media_type);

        return $media_type && $media_type->getSource() instanceof Jove;
      }
    }

    return FALSE;
  }

  /**
   * Validates width and height values.
   *
   * @param array $element
   *   Form element.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state.
   */
  public function elementValidateWidthHeight(array $element, FormStateInterface $form_state) {
    // Check if the value is a number with an optional decimal
    // or percentage sign, or "auto".
    if (!empty($element['#value']) && !preg_match('/^(auto|([0-9]*(\.[0-9]+)?%?))$/', $element['#value'])) {
      $form_state->setError(
        $element,
        $this->t(
          'The value entered for @dimension is invalid. Please insert a unitless integer for pixels, a percent, or "auto". Note that percent and auto may not function correctly depending on the browser and doctype.',
          ['@dimension' => $element['#title']]
        )
      );
    }
  }

}
