<?php

namespace Drupal\media_jove\Plugin\Validation\Constraint;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\media_jove\JoveUrlResolverInterface;
use Drupal\media_jove\Plugin\media\Source\Jove;
use GuzzleHttp\ClientInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;

/**
 * Validates JoVE URLs.
 */
class JoveEmbedCodeConstraintValidator extends ConstraintValidator implements ContainerInjectionInterface {

  /**
   * The oEmbed URL resolver service.
   *
   * @var \Drupal\media_jove\JoveUrlResolverInterface
   */
  protected $urlResolver;

  /**
   * The logger service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * Guzzle HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * Constructs a new JoveEmbedCodeConstraintValidator.
   *
   * @param \Drupal\media_jove\JoveUrlResolverInterface $url_resolver
   *   The oEmbed URL resolver service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger service.
   * @param \GuzzleHttp\ClientInterface $httpClient
   *   Guzzle HTTP client.
   */
  public function __construct(JoveUrlResolverInterface $url_resolver, LoggerChannelFactoryInterface $logger_factory, ClientInterface $httpClient) {
    $this->urlResolver = $url_resolver;
    $this->logger = $logger_factory->get('media');
    $this->httpClient = $httpClient;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('media_jove.url_resolver'),
      $container->get('logger.factory'),
      $container->get('http_client')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function validate($value, Constraint $constraint) {
    /** @var \Drupal\media\MediaInterface $media */
    $media = $value->getEntity();
    /** @var \Drupal\media\Plugin\media\Source\OEmbedInterface $source */
    $source = $media->getSource();

    if (!($source instanceof Jove)) {
      throw new \LogicException('Media source must be JoVE.');
    }

    $embedCode = $source->getSourceFieldValue($media);
    try {
      $id = $this->urlResolver->getVideoIdFromEmbedCode($embedCode);
      $response = $this->httpClient->request('GET', "https://www.jove.com/embed/directions/$id");

      if ($response->getStatusCode() != '200') {
        $this->context->addViolation($constraint->message);
      }
    }
    catch (\Exception $e) {
      $this->context->addViolation($constraint->message);
    }
  }

}
