<?php

namespace Drupal\media_jove\Plugin\media\Source;

use Drupal\Component\Utility\Html;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\Display\EntityFormDisplayInterface;
use Drupal\Core\Entity\Display\EntityViewDisplayInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldTypePluginManagerInterface;
use Drupal\Core\File\Exception\FileException;
use Drupal\Core\File\Exception\InvalidStreamWrapperException;
use Drupal\Core\File\FileExists;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\media\MediaInterface;
use Drupal\media\MediaSourceBase;
use Drupal\media\MediaSourceFieldConstraintsInterface;
use Drupal\media\MediaTypeInterface;
use Drupal\media_jove\JoveUrlResolverInterface;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\GuzzleException;
use GuzzleHttp\Exception\TransferException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * JoVE entity media source.
 *
 * @MediaSource(
 *   id = "jove",
 *   label = @Translation("JoVE"),
 *   allowed_field_types = {"string_long"},
 *   default_thumbnail_filename = "jove.png",
 *   description = @Translation("Provides business logic and metadata for JoVE.")
 * )
 */
class Jove extends MediaSourceBase implements MediaSourceFieldConstraintsInterface {

  /**
   * JoVE URL resolver.
   *
   * @var \Drupal\media_jove\JoveUrlResolverInterface
   */
  protected JoveUrlResolverInterface $resolver;

  /**
   * Drupal file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected FileSystemInterface $fileSystem;

  /**
   * Guzzle HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected ClientInterface $httpClient;

  /**
   * Drupal messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, EntityFieldManagerInterface $entity_field_manager, FieldTypePluginManagerInterface $field_type_manager, ConfigFactoryInterface $config_factory, JoveUrlResolverInterface $resolver, FileSystemInterface $fileSystem, ClientInterface $httpClient, MessengerInterface $messenger) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_type_manager, $entity_field_manager, $field_type_manager, $config_factory);

    $this->resolver = $resolver;
    $this->fileSystem = $fileSystem;
    $this->httpClient = $httpClient;
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('entity_field.manager'),
      $container->get('plugin.manager.field.field_type'),
      $container->get('config.factory'),
      $container->get('media_jove.url_resolver'),
      $container->get('file_system'),
      $container->get('http_client'),
      $container->get('messenger'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getSourceFieldConstraints() {
    return [
      'JoveEmbedCode' => [],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getMetadataAttributes() {
    return [
      'id' => $this->t('JoVE Video ID'),
      'title' => $this->t('JoVE Video title'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getMetadata(MediaInterface $media, $attribute_name) {
    $mediaUrl = $this->getSourceFieldValue($media);
    if (!$mediaUrl) {
      return parent::getMetadata($media, $attribute_name);
    }

    switch ($attribute_name) {
      case 'id':
        return $this->resolver->getVideoIdFromEmbedCode($mediaUrl);

      case 'thumbnail_uri':
        return $this->getLocalThumbnailUri($media) ?: parent::getMetadata($media, 'thumbnail_uri');

      case 'title':
        $id = $this->resolver->getVideoIdFromEmbedCode($mediaUrl);
        return $this->getVideoTitle($id) ?: "JoVE video {$media->id()}";

      default:
        break;
    }

    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function createSourceField(MediaTypeInterface $type) {
    return parent::createSourceField($type)->set('label', 'JoVE URL');
  }

  /**
   * {@inheritdoc}
   */
  public function getSourceFieldValue(MediaInterface $media) {
    $sourceField = $this->configuration['source_field'];
    if (empty($sourceField)) {
      throw new \RuntimeException('Source field for media source is not defined.');
    }

    /** @var \Drupal\Core\Field\FieldItemInterface $fieldItem */
    $fieldItem = $media->get($sourceField)->first();
    return $fieldItem->{$fieldItem::mainPropertyName()};
  }

  /**
   * {@inheritdoc}
   */
  public function prepareFormDisplay(MediaTypeInterface $type, EntityFormDisplayInterface $display) {
    parent::prepareFormDisplay($type, $display);
    $source_field = $this->getSourceFieldDefinition($type)->getName();

    $display->setComponent($source_field, [
      'type' => 'jove_textfield',
      'weight' => $display->getComponent($source_field)['weight'],
    ]);
    $display->removeComponent('name');
  }

  /**
   * {@inheritdoc}
   */
  public function prepareViewDisplay(MediaTypeInterface $type, EntityViewDisplayInterface $display) {
    $display->setComponent($this->getSourceFieldDefinition($type)->getName(), [
      'type' => 'jove_embed',
    ]);
  }

  /**
   * Returns the local URI for a video thumbnail.
   *
   * If the thumbnail is not already locally stored, this method will attempt
   * to download it.
   *
   * @param \Drupal\media\MediaInterface $media
   *   The media entity to get thumbnail URI for.
   *
   * @return string|null
   *   The local thumbnail URI, or NULL if it could not be downloaded.
   */
  protected function getLocalThumbnailUri(MediaInterface $media): ?string {
    $embedCode = $this->getSourceFieldValue($media);
    $id = $this->resolver->getVideoIdFromEmbedCode($embedCode);

    $localPath = $this->configFactory->get('system.file')->get('default_scheme') . '://media-jove/' . Html::escape($id) . '.jpg';
    if (!file_exists($localPath)) {
      try {
        $thumbnailUri = $this->getOriginalThumbnailUri($id);
        if ($thumbnailUri) {
          $dirname = $this->fileSystem->dirname($localPath);
          $this->fileSystem->prepareDirectory($dirname, FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS);
          $this->retrieveThumbnailFile($thumbnailUri, $localPath);
        }
      }
      catch (\Exception | GuzzleException $e) {
        return NULL;
      }
    }

    return $localPath;
  }

  /**
   * Returns URL to JoVE default thumbnail.
   *
   * @return string
   *   URL to thumbnail image.
   */
  protected function getOriginalThumbnailUri($id): string {
    $joveApiData = $this->getJoveApiData($id);

    // Note that below data structure might change, as it seems to be
    // an internal API.
    if (!empty($joveApiData['content']['domain']['headerImage'])) {
      return $joveApiData['content']['domain']['headerImage'];
    }
    return '';
  }

  /**
   * Returns title of the video by id.
   *
   * @param int|string $id
   *   Video id.
   *
   * @return string|null
   *   Video title if fetched.
   */
  protected function getVideoTitle(int|string $id): ?string {
    $joveApiData = $this->getJoveApiData($id);

    // Note that below data structure might change, as it seems to be
    // an internal API.
    if (!empty($joveApiData['content']['domain']['title'])) {
      // Strip HTML tags.
      $title = strip_tags($joveApiData['content']['domain']['title']);
      // Make sure it fits in the field.
      return mb_substr(trim($title), 0, 250);
    }
    return NULL;
  }

  /**
   * Retrieve thumbnail file (unmanaged).
   *
   * Replacement for deprecated system_retrieve_file() function.
   *
   * @param string $url
   *   The URL of the file to grab.
   * @param string|null $destination
   *   Stream wrapper URI specifying where the file should be placed. If a
   *   directory path is provided, the file is saved into that directory under
   *   its original name. If the path contains a filename as well, that one will
   *   be used instead.
   *   If this value is omitted, the site's default files scheme will be used,
   *   usually "public://".
   * @param int|\Drupal\Core\File\FileExists $replace
   *   Replace behavior when the destination file already exists:
   *   - FileSystemInterface::EXISTS_REPLACE: Replace the existing file.
   *   - FileSystemInterface::EXISTS_RENAME: Append _{incrementing number} until
   *     the filename is unique.
   *   - FileSystemInterface::EXISTS_ERROR: Do nothing and return FALSE.
   *
   * @return string|bool
   *   One of these possibilities:
   *   - If it succeeds the location where the file was saved.
   *   - If it fails, FALSE.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  protected function retrieveThumbnailFile(string $url, ?string $destination = NULL, int|FileExists $replace = FileExists::Replace): string|bool {
    try {
      $data = (string) $this->httpClient->get($url)->getBody();
      return $this->fileSystem->saveData($data, $destination, $replace);
    }
    catch (TransferException $exception) {
      $this->messenger->addError(t('Failed to fetch file due to error "%error"', ['%error' => $exception->getMessage()]));
    }
    catch (FileException | InvalidStreamWrapperException $e) {
      $this->messenger->addError(t('Failed to save file due to error "%error"', ['%error' => $e->getMessage()]));
    }
    return FALSE;
  }

  /**
   * Retrieve Jove metadata from (undocumented) API.
   *
   * @param string|int $id
   *   The media id.
   *
   * @return array|null
   *   Jove API data for this media id.
   */
  protected function getJoveApiData($id): ?array {
    $api_info_url = 'https://app.jove.com/api/free/article/en/' . $id;

    try {
      $response = $this->httpClient->request('GET', $api_info_url);

      if ($response->getStatusCode() == '200') {
        return json_decode($response->getBody(), TRUE);
      }

      return NULL;
    }
    catch (GuzzleException $exception) {
      return NULL;
    }

  }

}
